;;; "sql-null.scm" -- SQL NULL object and the ternary logic  -*- Scheme -*-

;;; Ivan Shmakov, 2007  This code is in public domain.

;;; Code:

(cond-expand
  (hygienic-macros

    (define-syntax sql-or
      (syntax-rules ()
        ((sql-or a ...)
         (sql-or/null #f a ...))))

    (define-syntax sql-or/null
      (syntax-rules ()
        ((sql-or/null null)
         null)
        ((sql-or/null null a b ...)
         (let ((ea a))
           (cond ((sql-null? ea) (sql-or/null ea    b ...))
                 ((not ea)       (sql-or/null null b ...))
                 (else           ea))))))

    (define-syntax sql-and
      (syntax-rules ()
        ((sql-and a ...)
         (sql-and/null #t a ...))))

    (define-syntax sql-and/null
      (syntax-rules ()
        ((sql-and/null null)
         null)
        ((sql-and/null null a b ...)
         (let ((ea a))
           (cond ((sql-null? ea) (sql-and/null ea    b ...))
                 (ea             (sql-and/null null b ...))
                 (else           ea)))))) )

  (else

    (define-macro (sql-or/null ?null . ?exprs)
      (if (null? ?exprs)
          `,?null
          (let ((?expr (car ?exprs))
                (?exprs (cdr ?exprs))
                (?expr-var (gensym)))
            `(let ((,?expr-var ,?expr))
               (cond ((sql-null? ,?expr-var)
                        (sql-or/null ,?expr-var ,@?exprs))
                      ((not ,?expr-var)
                        (sql-or/null ,?null ,@?exprs))
                      (else
                        ,?expr-var ) ) ) ) ) )

    (define-macro (sql-or . ?exprs)
      `(sql-or/null #f ,@?exprs) )

    (define-macro (sql-and/null ?null . ?exprs)
      (if (null? ?exprs)
          `,?null
          (let ((?expr (car ?exprs))
                (?exprs (cdr ?exprs))
                (?expr-var (gensym)))
            `(let ((,?expr-var ,?expr))
               (cond ((sql-null? ,?expr-var)
                        (sql-and/null ,?expr-var ,@?exprs))
                      (,?expr-var
                        (sql-and/null ,?null ,@?exprs))
                      (else
                        ,?expr-var ) ) ) ) ) )

    (define-macro (sql-and . ?exprs)
      `(sql-and/null #t ,@?exprs) ) ) )

;;; sql-null.scm ends here
